package org.xbib.interlibrary.client.bvb.order;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeFormatterBuilder;
import java.util.Locale;
import java.util.Objects;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.xbib.interlibrary.api.order.tracking.Event;
import org.xbib.interlibrary.api.order.tracking.EventHandler;
import org.xbib.interlibrary.http.HttpHelper;
import org.xbib.settings.Settings;

public class OrderTracker implements org.xbib.interlibrary.api.order.tracking.OrderTracker {

    private HttpHelper httpHelper;

    // pos, time, code
    private static final Pattern pattern =
            Pattern.compile("<div.*?><div.*?>(.*?)</div><div.*?>(.*?)</div><div.*?>(.*?)</div>", Pattern.MULTILINE);

    private static final DateTimeFormatter formatter = new DateTimeFormatterBuilder()
            .parseCaseInsensitive()
            .appendPattern("dd.MM.yyyy (HH:mm:ss)")
            .toFormatter(Locale.GERMAN);

    private String urlPatternOne;

    private String urlPatternTwo;

    public OrderTracker() {
    }

    @Override
    public String getISIL() {
        return "DE-604";
    }

    @Override
    public void init(Settings settings) throws IOException {
        this.httpHelper = new HttpHelper();
        httpHelper.open(settings);
        // ZFL-Nnummer
        this.urlPatternOne = Objects.requireNonNull(settings.get("url.singleparam"));
        // Sigel/ISIL + TAN
        this.urlPatternTwo = Objects.requireNonNull(settings.get("url.doubleparam"));
    }

    @Override
    public void track(String orderNumber, EventHandler eventHandler) throws IOException {
        String url = String.format(urlPatternOne, orderNumber.trim());
        httpHelper.get(url,
                params -> {},
                headers -> {},
                httpResponse -> {
                    String body = httpResponse.getBodyAsChars(StandardCharsets.UTF_8).toString();
                    parse(body, eventHandler);
                },
                exception -> {
                    throw new IllegalStateException(exception);
                },
                req -> {
                });
    }

    @Override
    public void track(String domain, String requesterId, String transactionId, EventHandler eventHandler) throws IOException {
        String url = String.format(urlPatternTwo, requesterId, transactionId.trim());
        httpHelper.get(url,
                params -> {},
                headers ->{},
                httpResponse -> {
                    String body = httpResponse.getBodyAsChars(StandardCharsets.UTF_8).toString();
                    parse(body, eventHandler);
                },
                exception -> {
                    throw new IllegalStateException(exception);
                },
                req -> {
                });
    }

    private void parse(String body, EventHandler eventHandler) {
        final Matcher matcher = pattern.matcher(body);
        while (matcher.find()) {
            final Event event = new Event() {

                @Override
                public Integer getPos() {
                    try {
                        return Integer.parseInt(matcher.group(1));
                    } catch (Exception e) {
                        return null;
                    }
                }

                @Override
                public LocalDateTime getTime() {
                    String time = matcher.group(2);
                    if (!time.isEmpty() && !time.isBlank()) {
                        return LocalDateTime.parse(time, formatter);
                    } else {
                        return null;
                    }
                }

                @Override
                public String getIsil() {
                    return null;
                }

                @Override
                public String getCode() {
                    String code = matcher.group(3);
                    // remove "span" markup
                    return code != null ? code.replaceAll("<[^>]*>","") : null;
                }

                @Override
                public String getInfo() {
                    return null;
                }
            };
            eventHandler.onEvent(event);
        }
    }
}
