package org.xbib.interlibrary.client.bvb.sru;

import java.util.Collection;
import java.util.List;
import java.util.Locale;
import java.util.function.Consumer;
import org.xbib.datastructures.common.MultiMap;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.interlibrary.action.avail.AvailRequest;
import org.xbib.interlibrary.action.avail.AvailResponse;
import org.xbib.interlibrary.action.avail.AvailResponseBuilder;
import org.xbib.interlibrary.action.avail.AvailServiceResponseBuilder;
import org.xbib.interlibrary.api.config.Configuration;
import org.xbib.interlibrary.api.library.Library;
import org.xbib.interlibrary.api.library.LibraryService;
import org.xbib.interlibrary.api.action.Response;
import org.xbib.interlibrary.api.library.DefaultLibraryService;
import org.xbib.interlibrary.api.action.Service;
import org.xbib.interlibrary.api.action.ServiceArguments;
import org.xbib.interlibrary.api.library.Material;
import org.xbib.interlibrary.api.library.Mode;
import org.xbib.interlibrary.sru.AbstractSRUAvailService;
import org.xbib.interlibrary.sru.DisplayHelper;
import org.xbib.interlibrary.sru.SRUClientHelper;
import org.xbib.marc.MarcRecord;
import org.xbib.marc.MarcRecordListener;
import static org.xbib.interlibrary.api.library.Material.ONLINE_RESOURCE;

public class SRUAvailService extends AbstractSRUAvailService {

    public SRUAvailService(ServiceArguments arguments, SRUClientHelper sruClientHelper) {
        super(arguments, sruClientHelper);
    }

    @Override
    protected String supports(String source) {
        return source != null && source.toLowerCase(Locale.ROOT).contains("bvb") ? "BVB" : null;
    }

    @Override
    protected MarcRecordListener getMarcRecordListener(AvailRequest availRequest,
                                                       AvailServiceResponseBuilder availResponse,
                                                       MultiMap<String, LibraryService> multiMap) {
        return new MarcRecordListener() {

            @Override
            public void beginCollection() {
            }

            @Override
            public void record(MarcRecord marcRecord) {
                if (availRequest.isBibliographicDescriptionEnabled()) {
                    availResponse.setBibliographicDescription(marcRecord);
                    availResponse.setDisplay(DisplayHelper.display(marcHelper, marcRecord, getName(), getName().toUpperCase(Locale.ROOT), null,
                            marcHelper.getRecordIdSupplier(marcRecord)));
                }
                extractBvbInterlibraryServices(configuration,
                        availRequest,
                        marcRecord, List.of("049"), multiMap);
                availResponse.setStatus(getName(), Response.Status.OK);
            }

            @Override
            public void endCollection() {
            }
        };
    }

    @Override
    public int compareTo(Service<AvailRequest, AvailResponse, AvailResponseBuilder> o) {
        return getName().compareTo(o.getName());
    }

    @Override
    protected String sourceIdCQL(String sourceId) {
        return "marcxml.idn = " + sourceId;
    }

    @Override
    protected Consumer<TinyMap.Builder<String, Object>> parameterCompletion() {
        return params -> {
            // startRecord not supported
            params.put("maximumRecords", 1);
        };
    }

    private void extractBvbInterlibraryServices(Configuration interlibraryConfiguration,
                                                AvailRequest availRequest,
                                                MarcRecord marcRecord,
                                                Collection<String> fields,
                                                MultiMap<String, LibraryService> multiMap) {
        List<Material> materials = marcHelper.extractMaterial(marcRecord);
        marcRecord.all(marcField -> fields.contains(marcField.getTag()), marcField -> {
            marcField.getSubfields().forEach(subfield -> {
                if ("a".equals(subfield.getId())) {
                    // ISIL - nothing else. So we have a problem. We solve it by looking up in our
                    // interlibrary configuration to create domain, organization, and ILL status.
                    // So, we do not have to configure domain especially for search action.
                    // The disadvantage is that we have to provide config action even if a server
                    // does not serve resolve or avail actions.
                    String isil = subfield.getValue();
                    Library library = interlibraryConfiguration.getLibrary(isil);
                    if (library != null) {
                        LibraryService.Builder libraryServiceBuilder = DefaultLibraryService.builder()
                                .setId(Integer.toString(random()))
                                .setISIL(library.getISIL())
                                .setName(library.getName())
                                .setDomainPriority(domainPriority(availRequest, library))
                                .setLibraryPriority(libraryPriority(availRequest, library))
                                .setMode(Mode.LOAN_COPY)
                                .setMaterials(materials)
                                .setServicePriority(materials.contains(ONLINE_RESOURCE) ? 1 : 2);
                        multiMap.put(isil, libraryServiceBuilder.build());
                    }
                }
            });
        });
    }
}
