package org.xbib.interlibrary.client.bvb.z;

import java.util.Locale;
import org.xbib.interlibrary.action.resolve.ResolveRequest;
import org.xbib.interlibrary.action.resolve.ResolveResponse;
import org.xbib.interlibrary.action.resolve.ResolveResponseBuilder;
import org.xbib.interlibrary.action.resolve.ResolveServiceResponseBuilder;
import org.xbib.interlibrary.api.action.Service;
import org.xbib.interlibrary.api.action.ServiceArguments;
import org.xbib.interlibrary.common.DefaultBibliographicDescription;
import org.xbib.interlibrary.z.AbstractZResolveService;
import org.xbib.marc.MarcField;
import org.xbib.marc.MarcRecord;

public class ZResolveService extends AbstractZResolveService {

    public ZResolveService(ServiceArguments arguments) {
        super(arguments);
    }

    @Override
    protected String supports(String source) {
        return source != null && source.toLowerCase(Locale.ROOT).contains("bvb") ? "BVB" : null;
    }

    @Override
    protected void resolve(ResolveRequest resolveRequest,
                           ResolveServiceResponseBuilder serviceResponseBuilder,
                           DefaultBibliographicDescription.Builder descriptionBuilder,
                           MarcRecord marcRecord) {
        for (MarcField marcField : marcRecord.getAll("025")) {
            String zdbID = marcField.getFirstSubfieldValue("z");
            if (zdbID != null) {
                descriptionBuilder.setSource("ZDB");
                descriptionBuilder.setSourceId(zdbID);
            }
        }
    }

    @Override
    protected String normalizeSourceId(String sourceId) {
        return sourceId;
    }

    @Override
    protected String getResolveCQL(String source, String sourceId) {
        // BIB-1 1=12 "local number"
        // BVB uses 53 ("number local call")
        // primitive quote here for DOI URLs, should work
        return "bib.numberLocalCall = \"" + sourceId + "\"";
    }

    @Override
    public int compareTo(Service<ResolveRequest, ResolveResponse, ResolveResponseBuilder> o) {
        return getName().compareTo(o.getName());
    }
}
